﻿local _,_,_,tocNum = GetBuildInfo();
local cata = false
if tocNum >= 40000 then
    cata = true
end
local myJSON = json.json
CHDMP = CHDMP or {}
local private = {}

local locale = CHDMP.Locales[GetLocale()]
if not locale then locale = CHDMP.Locales["enGB"] end
print(locale[11])

function private.GetGlobalInfo()
    local retTbl            = {}
    retTbl.locale           = GetLocale();
    retTbl.realm            = GetRealmName();
    retTbl.realmlist        = GetCVar("realmList");
    local version, build, d, tocversion = GetBuildInfo();
    retTbl.clientversion    = version;
    retTbl.clientbuild      = build;
    retTbl.clientdate       = d;
    retTbl.clienttocversion = tocversion;
    retTbl.dumptime         = time();
    retTbl.dumpversion      = GetAddOnMetadata("AldoranCharDump", "Version");
    return retTbl;
end
function private.GetUnitInfo()
    local retTbl            = {}
    retTbl.name             = UnitName("player");
    local _, class          = UnitClass("player");
    retTbl.class            = class;
    retTbl.level            = UnitLevel("player");
    local _,race            = UnitRace("player");
    retTbl.race             = race;
    retTbl.gender           = UnitSex("player");
    local honorableKills    = GetPVPLifetimeStats()
    retTbl.kills            = honorableKills;
    if not cata then
        retTbl.honor        = GetHonorCurrency();
        retTbl.arenapoints  = GetArenaCurrency();
    else
        retTbl.honor        = 0;
        retTbl.arenapoints  = 0;
    end
    retTbl.money            = GetMoney();
    retTbl.specs            = GetNumTalentGroups();
    return retTbl;
end
function private.GetSpellData()
    local retTbl = {}
    if not cata then
        for i = 1, MAX_SKILLLINE_TABS do
            local name, _, _, offset, numSpells = GetSpellTabInfo(i);
            if not name then
                break;
            end
            for s = offset + 1, offset + numSpells do
                spellInfo = GetSpellLink(s, BOOKTYPE_SPELL);
                if spellInfo then
                    for spellid in string.gmatch(spellInfo,".-Hspell:(%d+).*") do 
                        retTbl[spellid] = i;
                    end 
                end
            end
        end
    else
        for i = 1, 4 do
            local name, _, offset, numSpells, guildPerks = GetSpellTabInfo(i);
            if name and not guildPerks then
                for s = offset + 1, offset + numSpells do
                    spellInfo = GetSpellLink(s, BOOKTYPE_SPELL);
                    if spellInfo then
                        for spellid in string.gmatch(spellInfo,".-Hspell:(%d+).*") do 
                            retTbl[spellid] = i;
                        end 
                    end
                end
            end
        end
        -- 5 is guild perks
        for i = 6, 11 do
            name, _, _, _, numSpells, offset, _, _, _, _ = GetProfessionInfo(i)
            if name then
                for s = offset + 1, offset + numSpells do
                    spellInfo = GetSpellLink(s, BOOKTYPE_SPELL);
                    if spellInfo then
                        for spellid in string.gmatch(spellInfo,".-Hspell:(%d+).*") do 
                            retTbl[spellid] = i;
                        end 
                    end
                end
            end
        end
    end
    private.ILog(locale[1]); 
    return retTbl;
end
function private.GetGlyphData()
    local retTbl = {}
    if not cata then
        for i = 1, GetNumTalentGroups() do
            retTbl[i] = {[1] = {},[2] = {},[3] = {}}
            local curid = {[1] = 1,[2] = 1}
            for j = 1, 6 do
                local _, glyphType, glyphSpellID, _ = GetGlyphSocketInfo(j,i);
                if not retTbl[i][glyphType] then 
                    retTbl[i][glyphType] = {} 
                end
                retTbl[i][glyphType][curid[glyphType]] = glyphSpellID;
                curid[glyphType] = curid[glyphType]+1;
            end
        end
    else
        for i = 1, GetNumTalentGroups() do
            retTbl[i] = {[1] = {},[2] = {},[3] = {}}
            local curid = {[1] = 1,[2] = 1,[3] = 1}
            for j = 1, 9 do
                local _, glyphType, _, glyphSpellID, _ = GetGlyphSocketInfo(j,i);
                if not retTbl[i][glyphType] then 
                    retTbl[i][glyphType] = {} 
                end
                retTbl[i][glyphType][curid[glyphType]] = glyphSpellID;
                curid[glyphType] = curid[glyphType]+1;
            end
        end
    end
    private.ILog(locale[2]); 
    return retTbl;
end
function private.GetCurrencyData()
    local retTbl = {}
    if not cata then
        for i = 1, GetCurrencyListSize() do
            local name, _, _, _, _, count, _, _, itemID = GetCurrencyListInfo(i)
            retTbl[i] = {['C'] = count, ['I'] = itemID};
        end 
        private.ILog(locale[3]);
    else
        local wasError = false
        local dbc = CHDMP.CurrencyIds[GetLocale()]
        if not dbc then dbc = CHDMP.CurrencyIds["enGB"] end
        for i = 1, GetCurrencyListSize() do
            local name, isHeader, _, _, _, count, icon, _, _, _, _, itemID = GetCurrencyListInfo(i)
            if not isHeader then
                local id
                for k,v in pairs(dbc) do
                    local iconLast = (icon:match("[\\/]([^\\/]-)$") or ""):lower()
                    if name == v[1] and (iconLast == v[2]:lower() or (v[3] and iconLast == v[3]:lower())) then
                        id = k
                        break
                    end
                end
                if id then
                    retTbl[i] = {['C'] = count, ['I'] = id};
                else
                    wasError = true
                end
            end
        end 
        if wasError then
            private.ILog(locale[4]);
        else
            private.ILog(locale[3]);
        end
    end
    return retTbl;
end
function private.GetMACData()
    local retTbl = {}
    for i = 1, GetNumCompanions("MOUNT") do
        local _, _, M = GetCompanionInfo("MOUNT", i);
        retTbl["M:"..i] = M;
    end
    for i = 1, GetNumCompanions("CRITTER") do
        local _, _, C = GetCompanionInfo("CRITTER", i);
        retTbl["C:"..i] = C;
    end    
    private.ILog(locale[5]);    
    return retTbl;
end
function private.GetAchievements()
    local retTbl = {}
    local ids
    if not cata then
        ids = CHDMP.AchievementIds.PreCata
    else
        ids = CHDMP.AchievementIds.Cata
    end
    for _, j in pairs(ids) do
        IDNumber, _, _,Completed, Month, Day, Year, _, _, _, _, isGuildAch = GetAchievementInfo(j)
        if IDNumber and Completed and not isGuildAch then
            local posixtime = time{year = 2000 + Year, month = Month, day = Day};
            if posixtime then
                retTbl[IDNumber] = {["I"] = IDNumber, ["D"] = posixtime}
            end
        end
    end
    private.ILog(locale[6]);
    return retTbl;
end
function private.GetRepData()
    local retTbl = {}
    for i = 1, GetNumFactions() do 
        local name, _, _, _, _, earnedValue, _, canToggleAtWar, _, _, _, _, _ = GetFactionInfo(i)
        retTbl[i] = {["N"] = name, ["V"] = earnedValue, ["F"] = bit.bor(((not canToggleAtWar) and 16) or 0)} 
    end
    private.ILog(locale[7]);
    return retTbl;
end
function private.GetIData()
    local retTbl = {}
    for i = 1, 74 do 
        itemLink = GetInventoryItemLink("player", i) 
        if itemLink then 
            count = GetInventoryItemCount("player",i)
            if not cata then
                for entry, chant, Gem1, Gem2, Gem3, Gem4, SuffixID, UniqueID, lvl1 in string.gmatch(itemLink,".-Hitem:(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+).*") do 
                    retTbl["0000:"..i] =  {["I"] = entry, ["C"] = count, ["G1"] = Gem1, ["G2"] = Gem2, ["G3"] = Gem3, ["G4"] = Gem4, ["S"] = SuffixID, ["R"] = 0};
                end
            else
                for entry, chant, Gem1, Gem2, Gem3, Gem4, SuffixID, UniqueID, lvl1, ReforgeID in string.gmatch(itemLink,".-Hitem:(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+).*") do 
                    retTbl["0000:"..i] =  {["I"] = entry, ["C"] = count, ["G1"] = Gem1, ["G2"] = Gem2, ["G3"] = Gem3, ["G4"] = Gem4, ["S"] = SuffixID, ["R"] = ReforgeID};
                end
            end
        end 
    end
    for bag = 0, 11 do 
        for slot = 1, GetContainerNumSlots(bag) do 
            ItemLink = GetContainerItemLink(bag, slot) 
            if ItemLink then 
                local texture, count, locked, quality, readable = GetContainerItemInfo(bag, slot); 
                local Tbag = bag + 1000;
                if not cata then
                    for entry, chant, Gem1, Gem2, Gem3, Gem4, SuffixID, UniqueID, lvl1 in string.gmatch(ItemLink,".-Hitem:(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+).*") do 
                        retTbl[Tbag..":"..slot] = {["I"] = entry, ["C"] = count, ["G1"] = Gem1, ["G2"] = Gem2, ["G3"] = Gem3, ["G4"] = Gem4, ["S"] = SuffixID, ["R"] = 0};
                    end
                else
                    for entry, chant, Gem1, Gem2, Gem3, Gem4, SuffixID, UniqueID, lvl1, ReforgeID in string.gmatch(ItemLink,".-Hitem:(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+):(%-?%d+).*") do 
                        retTbl[Tbag..":"..slot] = {["I"] = entry, ["C"] = count, ["G1"] = Gem1, ["G2"] = Gem2, ["G3"] = Gem3, ["G4"] = Gem4, ["S"] = SuffixID, ["R"] = ReforgeID};
                    end
                end
            end 
        end 
    end
    private.ILog(locale[8]);    
    return retTbl;
end
function private.GetSkillData()
    local retTbl = {}
    for i = 1, GetNumSkillLines() do 
        local skillName, isHeader, _, skillRank, _, _, skillMaxRank, _, _, _, _, _, _ = GetSkillLineInfo(i)
        retTbl[i] = {["N"] = skillName,["C"] = skillRank,["M"] = skillMaxRank}
    end
    return retTbl;
end
function private.CreateCharDump()
    private.dmp             = private.dmp or {};
    private.dmp.ginf        = private.trycall(private.GetGlobalInfo, private.ErrLog)    or {};
    private.dmp.uinf        = private.trycall(private.GetUnitInfo, private.ErrLog)      or {};
    private.dmp.rep         = private.trycall(private.GetRepData, private.ErrLog)       or {};
    private.dmp.achiev      = private.trycall(private.GetAchievements, private.ErrLog)  or {};
    private.dmp.glyphs      = private.trycall(private.GetGlyphData, private.ErrLog)     or {};
    private.dmp.creature    = private.trycall(private.GetMACData, private.ErrLog)       or {};
    private.dmp.spells      = private.trycall(private.GetSpellData, private.ErrLog)     or {};
    if not cata then
        private.dmp.skills  = private.trycall(private.GetSkillData, private.ErrLog)     or {};
    else
        private.dmp.skills  = {};
    end
    private.dmp.inventory   = private.trycall(private.GetIData, private.ErrLog)         or {};
    private.dmp.currency    = private.trycall(private.GetCurrencyData, private.ErrLog)  or {};
    return b64_enc(myJSON.encode(private.dmp));
end
function private.Log(str_in)
    print("\124c0080C0FF  "..str_in.."\124r");
end
function private.ErrLog(err_in)
    private.errlog = private.errlog or ""
    private.errlog = private.errlog .. "err=" .. b64_enc(err_in) .. "\n"    
    print("\124c00FF0000"..(err_in or "nil").."\124r");
end
function private.GetCharDump()
    return b64_enc(private.CreateCharDump());
end
function private.ILog(str_in)
    print("\124c0080FF80"..str_in.."\124r");
end
function private.trycall(f,herr)
    local status, result = xpcall(f,herr)
    if status then 
        return result;
    end
    return status;
end
function private.SaveCharData(data_in)
    private.ILog(locale[9]);    
    CHDMP_DATA  = b64_enc(data_in)
    CHDMP_KEY   = Sha1(data_in)
end
function private.TradeSkillFrame_OnShow_Hook(frame, force)
    if private.done == true then
        return
    end
    if frame and frame.GetName and frame:GetName() == "TradeSkillFrame" then
        local isLink, _ = IsTradeSkillLinked();
        if isLink == nil then
            local link = GetTradeSkillListLink();
            if not link then
                return
            end
            local skillname = link:match("%[(.-)%]");
            private.dmp = private.dmp or {};
            private.dmp.skilllink = private.dmp.skilllink or {};
            private.dmp.skilllink[skillname] = link;
            print(format(locale[10], skillname, link))
            private.SaveCharData(b64_enc(private.GetCharDump()))
        end
    end 
end
SLASH_CHDMM1 = "/chardump";
SlashCmdList["CHDMM"] = function(msg)
    if msg == "done" then
        private.done = true;
        return;
    elseif msg == "help" then
        return;
    else
        private.done = false;
    end
    if not private.tradeskillframehooked then
        hooksecurefunc(_G, "ShowUIPanel", private.TradeSkillFrame_OnShow_Hook);
        private.tradeskillframehooked = true;
    end  
    private.SaveCharData(b64_enc(private.GetCharDump()))
end